#!/usr/local/bin/perl

# lbconfig
# Jeremy Kister 2011.02.15
# Released under Perl's Artistic License
# http://jeremy.kister.net./
#

use strict;

our %CONF;
BEGIN {
    %CONF = ( version => '20101123.01',
              homedir => '/home/hostedsites/lbconfig',
            );
    unshift( @INC, ("$CONF{homedir}/data/lib") );
};

use Time::HiRes qw(gettimeofday tv_interval);
use CGI qw(:standard -nosticky);
use CGI::Carp qw(fatalsToBrowser);
use A10::AX;
use A10::Misc;

my $q    = CGI->new;
my $colo = $q->param('colo');
my $zone = $q->param('zone');

open(CONFIG, "$CONF{homedir}/data/config") || die "cannot read $CONF{homedir}/data/config: $!\n";
while(<CONFIG>){
    s/#.*//g;
    next if(/^\s*$/);
    my($hname,$hcolo,$hzone,$hcolor,$hun,$hpw) = split /:/;
    $CONF{hosts}{$hname} = { zone => $hzone, color => $hcolor, colo => $hcolo, un => $hun, pw => $hpw };
}
close CONFIG;

my $misc  = A10::Misc->new(conf => \%CONF);
my $style = 'style="border-bottom: none; border-top: none;"';

print $q->header(-expires => '1h');
if( (my $output = $misc->getcache($colo,$zone)) && (! $q->param('nocache')) ){
    warn "[lbconfig] showing $colo/$zone cache to $ENV{REMOTE_ADDR}\n";
    for my $l ( @{$output} ){
        print $l;
    }
    exit;
}

my $t0 = [gettimeofday];
my $cache = "$CONF{homedir}/data/cache/";

my $uid = (stat( $cache ))[4];

unless( ($> == $<) && ($< == $uid) && (defined($uid)) ){
	die "cache directory $cache must be owned by the same user running this code.\n";
}

$cache .= $colo ? $colo : 'all';
$cache .= '.';
$cache .= $zone ? $zone : 'all';
open(my $fh, '>', "$cache.tmp") || die "cannot write to $cache.tmp: $!\n";

print $fh $q->start_html('A10-AX Config Viewer'), "\n",
          '<font size="2">', "\n",
          $misc->selector($colo,$zone), 
          '<table border="1" cellspacing="0">', "\n",
          $misc->banner;

for my $host ( sort { $a cmp $b || $a <=> $b } keys %{$CONF{hosts}} ){
    next if( $zone && ($CONF{hosts}{$host}{zone} ne $zone) );
    next if( $colo && ($CONF{hosts}{$host}{colo} ne $colo) );

    my $ax = A10::AX->new(conf => \%CONF, host => $host);
    my $servers = $ax->api('slb.server.getAll');
    my $groups  = $ax->api('slb.service-group.getAll');
    my $vips    = $ax->api('slb.vip.getAll');
    my $stats   = $ax->api('slb.server.fetchAllStatistics');

    # ip address-to-hostname map
    my %aton = map { $servers->{servers}{server}{$_}{address}, $_ } keys %{$servers->{servers}{server}};

    print $fh ' <tr><td colspan="15">', "\n",
              '  <table border="0" width="100%">', "\n",
              '   <tr bgcolor="#', $CONF{hosts}{$host}{color}, '"><td align="center">', $host, "</td></tr>\n",
              "  </table>\n",
              " </td></tr>\n";

    my $virt = $vips->{'virtual-servers'}->{'virtual-server'};
    for my $virtname (sort { $a cmp $b || $a <=> $b } keys %{$virt}){
        my $status  = $virt->{$virtname}{status};
        my $address = $virt->{$virtname}{address};       
        my $servref = $virt->{$virtname}{services}{service};

        # stupid API isnt consistent
        my $saref = (ref($servref) eq 'ARRAY') ? $servref : [ $servref ];

        my $bg = $misc->getcolor($status);

        print $fh " <tr>\n",
                  "  <td $bg>$virtname</td>\n",
                  "  <td $bg>$address</td>\n",
                  "  <td $bg>", $ax->status($status), "</td>\n";
        my $n = 0;
        for my $service ( @{$saref} ){
            my $bg = $misc->getcolor($service->{status});

            if($n){
                print $fh " <tr>\n",
                          "  <td $style>&nbsp;</td>\n" x 3;
            }

            print $fh "  <td $bg>$service->{port}</td>\n",
                      "  <td $bg>$service->{service_group}</td>\n",
                      "  <td $bg>", $ax->status($service->{status}), "</td>\n",
                      "  <td $bg>", $ax->type($service->{type}),     "</td>\n";
             
            # members of this group
            my $memref = $groups->{'service-groups'}{'service-group'}{ $service->{service_group} }{members}{member};
            # force results of API to be array (consitency)
            my $maref = (ref($memref) eq 'ARRAY') ? $memref : [ $memref ];
            
            my $i = 0;
            for my $member ( @{$maref} ){

                if($i){
                    print $fh " <tr>\n",
                              "  <td $style>&nbsp;</td>\n" x 9;
                }else{
                    my $shm = $groups->{'service-groups'}{'service-group'}{ $service->{service_group} }{'health_method'};
                    my $lbm = $groups->{'service-groups'}{'service-group'}{ $service->{service_group} }{'lb_method'};
                    my $lbeng = $ax->lb_method($lbm);
                    print $fh "  <td $bg>$shm</td>\n",
                              "  <td $bg>$lbeng</td>\n";
                }

                my $portref = $servers->{servers}{server}{ $aton{$member->{address}} }{ports}{port};
                my $paref = (ref($portref) eq 'ARRAY') ? $portref : [ $portref ];
                my $memhm;
                for my $port (@{$paref}){
                    if( $port->{port_num} == $member->{port} ){
                        $memhm = $port->{health_method};
                        last;
                    }
                }

                my $curr_conn = my $mbr_status = '*Unknown*';
                my $statref = $stats->{servers}{server}{ $aton{$member->{address}} }{ports}{port};
                my $saref = (ref($statref) eq 'ARRAY') ? $statref : [ $statref ];

                for my $aref (@{$saref}){
                    next unless($aref->{port_num} == $member->{port});
                    $curr_conn  = $aref->{cur_conns};
                    $mbr_status = $aref->{status};
                    last;
                }
                my $mbg = $misc->getcolor($mbr_status);

                print $fh "  <td $mbg>$aton{ $member->{address} }</td>\n",
                          "  <td $mbg>$member->{address}</td>\n",
                          "  <td $mbg>$member->{port}</td>\n",
                          "  <td $mbg>$memhm</td>\n",
                          "  <td $mbg>$curr_conn</td>\n",
                          "  <td $mbg>", $ax->status($mbr_status), "</td>\n",
    
                          " </tr>\n";
                $i=1;
            }

            $n=1;
        }
    }
    $ax->close;
}

my $date = localtime();

my $elapsed = tv_interval($t0);
warn "[lbconfig] generated $colo/$zone cache for $ENV{REMOTE_ADDR} in $elapsed sec\n";

print $fh $misc->banner, "\n",
          "</table>\n",
          "<br>\n",
          "Generated at $date GMT in $elapsed seconds.<br>\n",
          $q->end_html();
close $fh;

rename("$cache.tmp",$cache) || die "could not rename $cache.tmp => $cache: $!\n";

for my $l ( @{$misc->getcache($colo,$zone)} ){
    print $l;
}

